import 'package:analyzer/dart/analysis/results.dart';
import 'package:analyzer/dart/ast/ast.dart';
import 'package:analyzer/dart/ast/token.dart';
import 'package:analyzer/dart/ast/visitor.dart';
import 'package:analyzer_plugin/protocol/protocol_common.dart' as plugin;
import 'package:analyzer_plugin/protocol/protocol_generated.dart' as plugin;
import 'package:custom_code_analysis/logger/log.dart';
import 'package:custom_code_analysis/model/error_issue.dart';
import 'package:custom_code_analysis/model/rule.dart';
import 'package:uuid/uuid.dart';

class ClickableWidgetIdMissing extends Rule {
  final CompilationUnit _compilationUnit;
  final ResolvedUnitResult analysisResult;
  String unitPath;

  static const ruleId = 'clickable_widget_id_missing';
  static const _comment = '快速添加';
  static const className = 'ClickableWidget';
  static const requiredField = 'id';
  static const _message = '$className缺少$requiredField参数';
  static const _correction = '$requiredField为$className必备参数，点击"$_comment"一键修复';

  ClickableWidgetIdMissing(this._compilationUnit, this.analysisResult) {
    unitPath = this._compilationUnit.declaredElement.source.fullName;
    logUtil.info("checker $unitPath");
  }

  int _getLineNumber(Token token) {
    return _compilationUnit.lineInfo.getLocation(token.offset).lineNumber;
  }

  // int _getColumnNumber(Token token) {
  //   return _compilationUnit.lineInfo.getLocation(token.offset).columnNumber;
  // }

  // String _whiteSpace(int count) {
  //   var _result = '';
  //   for (int i = 0; i < count; i++) {
  //     _result += ' ';
  //   }
  //   return _result;
  // }

  String _generateReplacement(ArgumentList node) {
    int left = _getLineNumber(node.leftParenthesis);
    int right = _getLineNumber(node.rightParenthesis);
    // int offsetLeft = _getColumnNumber(node.leftParenthesis);
    // int offsetRight = _getColumnNumber(node.rightParenthesis);
    String result;
    String _randomId = Uuid().v4().toString().replaceAll('-', '');

    String content = analysisResult.content;
    if (left == right) {
      result = '${content.substring(node.offset, node.offset + 1)}$requiredField: \'$_randomId\', ${content.substring(node.offset + 1, node.end)}';
    } else {
      // result =
      //     '${content.substring(node.offset, node.offset + 1)}\n${_whiteSpace(_compilationUnit.lineInfo.getLocation(node.parent.offset).columnNumber + 2)}name: \'$_randomId\', ${content.substring(node.offset + 1, node.end)}';
      result = '${content.substring(node.offset, node.offset + 1)}$requiredField: \'$_randomId\', ${content.substring(node.offset + 1, node.end)}';
    }
    // logUtil.info(
    //     'row = ($left, $right), col = ($offsetLeft, $offsetRight), offset = (${node.leftParenthesis.offset}, ${node.rightParenthesis.offset}) result = $result');
    return result;
  }

  List<ErrorIssue> errors() {
    final visitor = _MirrorVisitor();
    _compilationUnit.accept(visitor);
    return visitor.nodes.map(
      (node) {
        // node.arguments.add(astFactory.assignmentExpression("$requiredField", TokenType.AMPERSAND, "fdf"));
        return ErrorIssue(
          error: plugin.AnalysisError(
            plugin.AnalysisErrorSeverity.INFO,
            plugin.AnalysisErrorType.HINT,
            plugin.Location(
              _compilationUnit.declaredElement.source.fullName,
              node.offset,
              node.length,
              _compilationUnit.lineInfo.getLocation(node.offset).lineNumber,
              _compilationUnit.lineInfo.getLocation(node.offset).columnNumber,
            ),
            _message,
            'code',
            correction: _correction,
            hasFix: true,
          ),
          replacement: _generateReplacement(node),
          fixes: codeIssueToAnalysisErrorFixes2(
            ErrorIssue(
              error: plugin.AnalysisError(
                plugin.AnalysisErrorSeverity.INFO,
                plugin.AnalysisErrorType.HINT,
                plugin.Location(
                  _compilationUnit.declaredElement.source.fullName,
                  node.offset,
                  node.length,
                  _compilationUnit.lineInfo.getLocation(node.offset).lineNumber,
                  _compilationUnit.lineInfo.getLocation(node.offset).columnNumber,
                ),
                _message,
                'code',
                correction: _correction,
                hasFix: true,
              ),
              replacement: _generateReplacement(node),
            ),
            analysisResult,
          ),
        );
      },
    ).toList();
  }

// void printMembers(CompilationUnit unit) {
// for (CompilationUnitMember unitMember in unit.declarations) {
//   if (unitMember is ClassDeclaration) {
//     logUtil.info('unitMember.name.name= $unitMember.name.name');
//     for (ClassMember classMember in unitMember.members) {
//       if (classMember is MethodDeclaration) {
//         logUtil.info('classMember.name  ${classMember.name}');
//       } else if (classMember is FieldDeclaration) {
//         for (VariableDeclaration field in classMember.fields.variables) {
//           logUtil.info('field.name.name  ${field.name.name}');
//         }
//       } else if (classMember is ConstructorDeclaration) {
//         if (classMember.name == null) {
//           logUtil.info('unitMember.name.name  ${unitMember.name.name}');
//         } else {
//           logUtil.info('unitMember.name.name.classMember.name.name  ${unitMember.name.name}.${classMember.name.name}');
//         }
//       }
//     }
//   }
// }
// }

  // plugin.AnalysisErrorFixes codeIssueToAnalysisErrorFixes(plugin.AnalysisError error, ResolvedUnitResult unitResult) {
  //   return plugin.AnalysisErrorFixes(
  //     error,
  //     fixes: [
  //       if (error.correction != null)
  //         plugin.PrioritizedSourceChange(
  //           99,
  //           plugin.SourceChange(
  //             _comment,
  //             edits: [
  //               plugin.SourceFileEdit(
  //                 unitResult.libraryElement.source.fullName,
  //                 unitResult.libraryElement.source.modificationStamp,
  //                 edits: [
  //                   plugin.SourceEdit(
  //                     error.location.offset,
  //                     error.location.length,
  //                     '_generateReplacement()',
  //                   ),
  //                 ],
  //               ),
  //             ],
  //             // selection: Position(unitResult.libraryElement.source.fullName, issue.offset),
  //           ),
  //         ),
  //     ],
  //   );
  // }

  plugin.AnalysisErrorFixes codeIssueToAnalysisErrorFixes2(ErrorIssue error, ResolvedUnitResult unitResult) {
    // logUtil.info('error = ${error.toString()}');
    return plugin.AnalysisErrorFixes(
      error.error,
      fixes: [
        if (error.error.correction != null)
          plugin.PrioritizedSourceChange(
            99,
            plugin.SourceChange(
              _comment,
              edits: [
                plugin.SourceFileEdit(
                  unitResult.libraryElement.source.fullName,
                  unitResult.libraryElement.source.modificationStamp,
                  edits: [
                    plugin.SourceEdit(
                      error.error.location.offset,
                      error.error.location.length,
                      error.replacement,
                    ),
                  ],
                ),
              ],
              // selection: Position(unitResult.libraryElement.source.fullName, issue.offset),
            ),
          ),
      ],
    );
  }
}

class _MirrorVisitor extends RecursiveAstVisitor<void> {
  final _nodes = <ArgumentList>[];

  Iterable<ArgumentList> get nodes => _nodes;

  @override
  void visitArgumentList(ArgumentList node) {
    super.visitArgumentList(node);

    if (node.parent.childEntities.isNotEmpty) {
      if (node.parent.childEntities.toString().contains(ClickableWidgetIdMissing.className)) {
        bool _isNeedFix = true;
        for (final item in node.arguments) {
          if (item.toString().contains(ClickableWidgetIdMissing.requiredField)) {
            _isNeedFix = false;
            break;
          }
        }
        if (_isNeedFix) {
          _nodes.add(node);
        }
      }
    }
  }
}
